package com.engine.salary.elog.config;

import com.engine.salary.elog.entity.dto.LoggerContext;
import com.engine.salary.mapper.elog.ElogTableCheckerMapper;
import com.engine.salary.elog.util.db.IdGenerator;
import com.engine.salary.elog.util.db.MapperProxyFactory;


public class ELogTableChecker {
    private static final long version = 0;
    private ELogCache eLogCache = new ELogCache();

    private ElogTableCheckerMapper getTableCheckerMapper() {
        return MapperProxyFactory.getProxy(ElogTableCheckerMapper.class);
    }

    public <T> void check(LoggerContext<T> loggerContext) {
        String module = loggerContext.getModuleName();
        String function = loggerContext.getFunctionName();
        String mainTable = String.format("%s_%slogs", module, function);
        Long v = eLogCache.getVersion(mainTable);
        boolean noTable = v == null;
        if (noTable) {
            this.initTable(mainTable);
            v = version;
        } else {
            this.checkVersion(mainTable, v);
        }

    }

    private void checkVersion(String mainTable, Long v) {
        if (v == version) {
            return;
        }
    }


    private void initTable(String mainTable) {
        getTableCheckerMapper().recordVersion(IdGenerator.generate(), mainTable, version);
        getTableCheckerMapper().createMainTable(mainTable);
        getTableCheckerMapper().createDetailTable(mainTable + "_detail");
    }
}
