package com.engine.salary.elog.entity.dto;

import com.alibaba.fastjson.annotation.JSONField;
import com.engine.salary.elog.annotation.ElogField;
import com.engine.salary.elog.annotation.ElogTable;
import com.engine.salary.elog.enums.ElogConsts;
import com.engine.salary.elog.annotation.ApiModel;
import com.engine.salary.elog.annotation.ApiModelProperty;
import org.apache.commons.lang3.StringUtils;
import weaver.hrm.User;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: LoggerContext
 * @Description 日志实体类。支持通过泛型扩展日志字段
 * @Author tanghj
 * @Date 2021/2/10 14:18
 */
@ElogTable(module = ElogConsts.BASE_TABLE)
@ApiModel("日志实体类")
public class LoggerContext<T> implements Serializable {

    private static final long serialVersionUID = 15869325700230992L;

    private User user;

    @ElogField(comment = "ID", dataType = DataTypeEnum.BIGINT, isKey = true)
    @ApiModelProperty("日志ID")
    private long id;

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 36, comment = "日志UUID")
    @ApiModelProperty("日志UUID")
    private String uuid = "";

    @ApiModelProperty("自定义日志字段信息")
    private T customInfo;
    @ElogField(dataType = DataTypeEnum.DATETIME, comment = "操作时间")
    @ApiModelProperty("操作时间")
    @JSONField(format = "yyyy-MM-dd HH:mm:ss")
    private Date date = new Date();

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 500, comment = "终端信息")
    @ApiModelProperty("终端信息")
    private String device = "";

    @ElogField(dataType = DataTypeEnum.BIGINT, defaultValue = "-1", comment = "操作人")
    @ApiModelProperty("操作人")
    private String operator = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 100, comment = "操作人姓名")
    @ApiModelProperty("操作人姓名")
    private String operatorName = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 10, comment = "租户id")
    @ApiModelProperty("租户id")
    private String tenant_key = "";

    /**
     * 要操作的对象在表中的主键值
     */
    @ElogField(dataType = DataTypeEnum.BIGINT, defaultValue = "-1", comment = "操作目标id")
    @ApiModelProperty("操作目标id")
    private String targetId = "";

    @ElogField(dataType = DataTypeEnum.TEXT, comment = "操作目标名称")
    @ApiModelProperty("操作目标名称（用于显示）")
    private String targetName = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 100, comment = "模块")
    @ApiModelProperty("目标对象类型（大分类，模块，服务）")
    private String moduleName;//  模块

    /**
     * 目标对象类型（小分类，模块/服务下的子功能。子项目）
     * 数据存储是以模块名_子项目名作为最基本的存储单元
     * 如果是子项目下的子项目 命名为：子项目名_子项目名_子项目名（全部小写）
     */
    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 100, comment = "服务（方法）")
    @ApiModelProperty("目标对象类型（小分类，模块/服务下的子功能。子项目）")
    private String functionName;

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 100, comment = "访问接口名")
    @ApiModelProperty("访问接口名")
    private String interfaceName = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 200, comment = "请求全路径")
    @ApiModelProperty("请求全路径")
    private String requestUrl = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 200, comment = "请求地址")
    @ApiModelProperty("请求地址")
    private String requestUri = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 50, comment = "操作类型")
    @ApiModelProperty("操作类型（增删改查等）")
    private String operateType = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 100, comment = "操作类型名称")
    @ApiModelProperty("操作类型名称")
    private String operateTypeName = "";

    /**
     * 每个TableChangeBean 为一张表，支持记录多张表的前后值
     */
    @ApiModelProperty("修改前、后的值")
    private List<TableChangeBean> changeValues;// 操作表名,[字段名,值]

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 3000, comment = "操作详细说明")
    @ApiModelProperty("操作详细说明")
    private String operatedesc = "";

    @ElogField(dataType = DataTypeEnum.LONGTEXT, comment = "涉及的相关参数")
    @ApiModelProperty("涉及的相关参数")
    private Map<String, Object> params;

    @ApiModelProperty("涉及的相关参数-转string存储")
    private String paramsStr;

    /**
     * 当作为主表，belongMainId不赋值
     */
    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 36, comment = "所属主表uuid")
    @ApiModelProperty("所属主表uuid")
    private String belongMainId = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 50, comment = "操作IP")
    @ApiModelProperty("操作IP")
    private String clientIp = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 50, comment = "分组")
    @ApiModelProperty("分组")
    private String groupId = "";

    /*@ApiModelProperty("是否明显表")
    private boolean isDetail;*/

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 1000, comment = "分组标题")
    @ApiModelProperty("分组标题")
    private String groupNameLabel = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 200, comment = "重做业务接口")
    @ApiModelProperty("重做业务接口")
    private String redoService = "";

    @ElogField(dataType = DataTypeEnum.LONGTEXT, comment = "重做参数")
    @ApiModelProperty("重做参数")
    private RedoContext redoContext;

    @ApiModelProperty("重做参数-转String存储")
    private String redoContextStr;

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 200, comment = "撤销业务接口")
    @ApiModelProperty("撤销业务接口")
    private String cancelService = "";

    @ElogField(dataType = DataTypeEnum.LONGTEXT, comment = "撤销参数")
    @ApiModelProperty("撤销参数")
    private CancelContext cancelContext;

    @ApiModelProperty("撤销参数-转String存储")
    private String cancelContextStr;

    @ApiModelProperty("日志明细列表（值变化列表-自动赋值、或者自定义字段）")
    private List<LoggerDetailContext> detailContexts;

    @ElogField(dataType = DataTypeEnum.DATETIME, defaultValue = "CURRENT_TIMESTAMP", comment = "创建时间")
    @ApiModelProperty("创建时间")
    @JSONField(format = "yyyy-MM-dd HH:mm:ss")
    private Date create_time;

    @ElogField(dataType = DataTypeEnum.DATETIME, defaultValue = "CURRENT_TIMESTAMP", comment = "修改时间")
    @ApiModelProperty("修改时间")
    @JSONField(format = "yyyy-MM-dd HH:mm:ss")
    private Date update_time;

    @ElogField(dataType = DataTypeEnum.BIGINT, defaultValue = "-1", comment = "创建人id")
    @ApiModelProperty("创建人id")
    private long creator;

    @ElogField(dataType = DataTypeEnum.INT, defaultValue = "0", comment = "是否删除")
    @ApiModelProperty("是否删除")
    private int delete_type;

    @ElogField(dataType = DataTypeEnum.BIGINT, defaultValue = "0", comment = "总运行时长")
    @ApiModelProperty("总运行时长")
    private long totalRunTime;

    @ElogField(dataType = DataTypeEnum.BIGINT, defaultValue = "0", comment = "主方法运行时长")
    @ApiModelProperty("主方法运行时长")
    private long mainRunTime;

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 100, defaultValue = "", comment = "运行结果标识")
    @ApiModelProperty("运行结果标识")
    private String result = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 100, defaultValue = "", comment = "来自pc web")
    @ApiModelProperty("来自终端")
    private String fromTerminal = "";

    @ElogField(dataType = DataTypeEnum.TEXT, comment = "运行结果描述")
    @ApiModelProperty("运行结果描述")
    private String resultDesc = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 3000, comment = "原先内容")
    @ApiModelProperty("原先内容（et用）")
    private String old_content = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 20, comment = "链接类型")
    @ApiModelProperty("链接类型（et用）")
    private String link_type = "";

    @ElogField(dataType = DataTypeEnum.BIGINT, defaultValue = "0", comment = "链接id")
    @ApiModelProperty("链接id（et用）")
    private long link_id;

    @ElogField(dataType = DataTypeEnum.BIGINT, defaultValue = "0", comment = "原先链接id")
    @ApiModelProperty("原先链接id（et用）")
    private long old_link_id;

    /**
     * 开发可以自由传中间临时参数，不会入库
     */
    @ApiModelProperty("临时参数")
    private Object tempParams;

    /**
     * 是否忽略该日志，不进行数据记录
     */
    @ApiModelProperty("是否忽略该日志，不进行数据记录")
    private boolean logIgnore;

    /**
     * 审计操作类型
     */
    @ApiModelProperty("审计操作类型")
    protected String operateAuditType;

    /**
     * 操作人账号
     */
    @ApiModelProperty("操作人账号")
    protected String operateAccount;

    @ApiModelProperty("操作人id（兼容数据库用）")
    protected long logOperator;

    @ApiModelProperty("操作人id（兼容数据库用）")
    protected long logTargetid;

    @ApiModelProperty("ES存储大字段功能标识")
    protected Boolean esFunction;

    @ApiModelProperty("params参数是否忽略记录")
    private Boolean paramsIgnore = Boolean.FALSE;

    @ApiModelProperty("params请求体keys")
    private List<String> paramsBodyKeys;

    @ApiModelProperty("日志弱控记录")
    private Boolean weakElogReocrd = Boolean.FALSE;

    private List<String> clobFieldList;

    public User getUser() {
        return user;
    }

    public void setUser(User user) {
        this.user = user;
    }

    public List<String> getClobFieldList() {
        return clobFieldList;
    }

    public void setClobFieldList(List<String> clobFieldList) {
        this.clobFieldList = clobFieldList;
    }

    public Boolean getWeakElogReocrd() {
        return weakElogReocrd;
    }

    public void setWeakElogReocrd(Boolean weakElogReocrd) {
        this.weakElogReocrd = weakElogReocrd;
    }

    public Boolean getParamsIgnore() {
        return paramsIgnore;
    }

    public void setParamsIgnore(Boolean paramsIgnore) {
        this.paramsIgnore = paramsIgnore;
    }

    public List<String> getParamsBodyKeys() {
        return paramsBodyKeys;
    }

    public void setParamsBodyKeys(List<String> paramsBodyKeys) {
        this.paramsBodyKeys = paramsBodyKeys;
    }

    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public T getCustomInfo() {
        return customInfo;
    }

    public void setCustomInfo(T customInfo) {
        this.customInfo = customInfo;
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public String getDevice() {
        return device;
    }

    public void setDevice(String device) {
        this.device = device;
    }

    public String getOperator() {
        return operator;
    }

    public void setOperator(String operator) {
        this.operator = operator;
    }

    public String getTenant_key() {
        return tenant_key;
    }

    public void setTenant_key(String tenant_key) {
        this.tenant_key = tenant_key;
    }

    public String getTargetId() {
        return targetId;
    }

    public void setTargetId(String targetId) {
        this.targetId = targetId;
    }

    public String getTargetName() {
        return targetName;
    }

    public void setTargetName(String targetName) {
        this.targetName = targetName;
    }

    public String getModuleName() {
        return moduleName;
    }

    public void setModuleName(String moduleName) {
        this.moduleName = moduleName;
    }

    public String getFunctionName() {
        return functionName;
    }

    public void setFunctionName(String functionName) {
        this.functionName = functionName;
    }

    public String getInterfaceName() {
        return interfaceName;
    }

    public void setInterfaceName(String interfaceName) {
        this.interfaceName = interfaceName;
    }

    public String getOperateType() {
        return operateType;
    }

    public void setOperateType(String operateType) {
        this.operateType = operateType;
    }

    public List<TableChangeBean> getChangeValues() {
        return changeValues;
    }

    public void setChangeValues(List<TableChangeBean> changeValues) {
        this.changeValues = changeValues;
    }

    public String getOperatedesc() {
        return operatedesc;
    }

    public void setOperatedesc(String operatedesc) {
        this.operatedesc = operatedesc;
    }

    public Map<String, Object> getParams() {
        return params;
    }

    public void setParams(Map<String, Object> params) {
        this.params = params;
    }

    /*public String getMainId() {
        return mainId;
    }

    public void setMainId(String mainId) {
        this.mainId = mainId;
    }

    public String getBelongMainId() {
        return belongMainId;
    }

    public void setBelongMainId(String belongMainId) {
        this.belongMainId = belongMainId;
    }*/

    public String getClientIp() {
        return clientIp;
    }

    public void setClientIp(String clientIp) {
        this.clientIp = clientIp;
    }

    public String getGroupId() {
        return groupId;
    }

    public void setGroupId(String groupId) {
        this.groupId = groupId;
    }

/*    public boolean isDetail() {
        return isDetail;
    }

    public void setDetail(boolean detail) {
        isDetail = detail;
    }*/

    public String getGroupNameLabel() {
        return groupNameLabel;
    }

    public void setGroupNameLabel(String groupNameLabel) {
        this.groupNameLabel = groupNameLabel;
    }

    public String getRedoService() {
        return redoService;
    }

    public void setRedoService(String redoService) {
        this.redoService = redoService;
    }

    public RedoContext getRedoContext() {
        return redoContext;
    }

    public void setRedoContext(RedoContext redoContext) {
        this.redoContext = redoContext;
    }

    public CancelContext getCancelContext() {
        return cancelContext;
    }

    public void setCancelContext(CancelContext cancelContext) {
        this.cancelContext = cancelContext;
    }

    public String getCancelService() {
        return cancelService;
    }

    public void setCancelService(String cancelService) {
        this.cancelService = cancelService;
    }

    public List<LoggerDetailContext> getDetailContexts() {
        return detailContexts;
    }

    public void setDetailContexts(List<LoggerDetailContext> detailContexts) {
        this.detailContexts = detailContexts;
    }

    public void addDetailContext(LoggerDetailContext loggerDetailContext) {
        if (this.detailContexts != null) {
            this.detailContexts.add(loggerDetailContext);
        } else {
            this.detailContexts = new ArrayList<>();
            this.detailContexts.add(loggerDetailContext);
        }
    }

    public void addDetailContext(List<LoggerDetailContext> list) {
        if (this.detailContexts != null) {
            this.detailContexts.addAll(list);
        } else {
            this.detailContexts = new ArrayList<>();
            this.detailContexts.addAll(list);
        }
    }

    public String getOperatorName() {
        return operatorName;
    }

    public void setOperatorName(String operatorName) {
        this.operatorName = operatorName;
    }

    public void setOldValues(Object object) {
        TableChangeBean bean = new TableChangeBean();
        bean.setOldValue(object);
        getChangeList().add(bean);
    }

    public void setOldValueList(List<Object> list) {
        if (list != null)
            list.stream().forEach(obj -> setOldValues(obj));
    }

    public void setNewValueList(List<Object> list) {
        if (list != null)
            list.stream().forEach(obj -> setNewValues(obj));
    }

    public void setNewValues(Object object) {
        List<TableChangeBean> list = getChangeList();

        boolean handled = false;
        for (TableChangeBean bean : list) {
            if (bean.getNewValue() == null) {
                bean.setNewValue(object);
                handled = true;
                break;
            }
        }
        if (!handled) {
            TableChangeBean bean = new TableChangeBean();
            bean.setNewValue(object);
            list.add(bean);
        }
    }

    public void setOldValues(Object object, String tableName, String dataId, String belongDataid) {
        setOldValues(object, tableName, dataId, belongDataid, false);
    }

    public void setOldValues(Object object, String tableName, String dataId, String belongDataid, boolean isDetail) {
        TableChangeBean bean = new TableChangeBean();
        if (StringUtils.isNotEmpty(tableName))
            bean.setTableName(tableName);
        if (StringUtils.isNotEmpty(dataId))
            bean.setDataid(dataId);
        if (StringUtils.isNotEmpty(belongDataid))
            bean.setBelongDataid(belongDataid);
        if (isDetail)
            bean.setIsDetail(1);
        bean.setOldValue(object);
        getChangeList().add(bean);
    }

    public void setOldValueList(List<Object> list, String tableName, String dataId, String belongDataid) {
        setOldValueList(list, tableName, dataId, belongDataid, false);
    }

    public void setNewValueList(List<Object> list, String tableName, String dataId, String belongDataid) {
        setNewValueList(list, tableName, dataId, belongDataid, false);
    }

    public void setOldValueList(List<Object> list, String tableName, String dataId, String belongDataid, boolean isDetail) {
        if (list != null)
            list.stream().forEach(obj -> setOldValues(obj, tableName, dataId, belongDataid, isDetail));
    }

    public void setNewValueList(List<Object> list, String tableName, String dataId, String belongDataid, boolean isDetail) {
        if (list != null)
            list.stream().forEach(obj -> setNewValues(obj, tableName, dataId, belongDataid, isDetail));
    }

    public void setNewValues(Object object, String tableName, String dataId, String belongDataid) {
        setNewValues(object, tableName, dataId, belongDataid, false);
    }

    public void setNewValues(Object object, String tableName, String dataId, String belongDataid, boolean isDetail) {
        List<TableChangeBean> list = getChangeList();

        boolean handled = false;
        for (TableChangeBean bean : list) {
            if (bean.getNewValue() == null) {
                bean.setNewValue(object);
                if (StringUtils.isNotEmpty(tableName))
                    bean.setTableName(tableName);
                if (StringUtils.isNotEmpty(dataId))
                    bean.setDataid(dataId);
                if (StringUtils.isNotEmpty(belongDataid))
                    bean.setBelongDataid(belongDataid);
                if (isDetail)
                    bean.setIsDetail(1);
                handled = true;
                break;
            }
        }
        if (!handled) {
            TableChangeBean bean = new TableChangeBean();
            if (StringUtils.isNotEmpty(tableName))
                bean.setTableName(tableName);
            if (StringUtils.isNotEmpty(dataId))
                bean.setDataid(dataId);
            if (StringUtils.isNotEmpty(belongDataid))
                bean.setBelongDataid(belongDataid);
            if (isDetail)
                bean.setIsDetail(1);
            bean.setNewValue(object);
            list.add(bean);
        }
    }

    public List<TableChangeBean> getChangeList() {

        if (this.changeValues == null) {
            this.changeValues = new ArrayList<>();
        }

        return this.changeValues;

    }

    public Object getParam(String id) {
        return this.params.get(id);
    }

    public String getOperateTypeName() {
        return operateTypeName;
    }

    public void setOperateTypeName(String operateTypeName) {
        this.operateTypeName = operateTypeName;
    }

    public String getParamsStr() {
        return paramsStr;
    }

    public void setParamsStr(String paramsStr) {
        this.paramsStr = paramsStr;
    }

    public String getRedoContextStr() {
        return redoContextStr;
    }

    public void setRedoContextStr(String redoContextStr) {
        this.redoContextStr = redoContextStr;
    }

    public String getCancelContextStr() {
        return cancelContextStr;
    }

    public void setCancelContextStr(String cancelContextStr) {
        this.cancelContextStr = cancelContextStr;
    }

    public String getBelongMainId() {
        return belongMainId;
    }

    public void setBelongMainId(String belongMainId) {
        this.belongMainId = belongMainId;
    }

    public String getRequestUrl() {
        return requestUrl;
    }

    public void setRequestUrl(String requestUrl) {
        this.requestUrl = requestUrl;
    }

    public String getRequestUri() {
        return requestUri;
    }

    public void setRequestUri(String requestUri) {
        this.requestUri = requestUri;
    }

    public Date getCreate_time() {
        return create_time;
    }

    public void setCreate_time(Date create_time) {
        this.create_time = create_time;
    }

    public Date getUpdate_time() {
        return update_time;
    }

    public void setUpdate_time(Date update_time) {
        this.update_time = update_time;
    }

    public long getCreator() {
        return creator;
    }

    public void setCreator(long creator) {
        this.creator = creator;
    }

    public int getDelete_type() {
        return delete_type;
    }

    public void setDelete_type(int delete_type) {
        this.delete_type = delete_type;
    }

    public long getTotalRunTime() {
        return totalRunTime;
    }

    public void setTotalRunTime(long totalRunTime) {
        this.totalRunTime = totalRunTime;
    }

    public long getMainRunTime() {
        return mainRunTime;
    }

    public void setMainRunTime(long mainRunTime) {
        this.mainRunTime = mainRunTime;
    }

    public Object getTempParams() {
        return tempParams;
    }

    public void setTempParams(Object tempParams) {
        this.tempParams = tempParams;
    }

    public String getResult() {
        return result;
    }

    public void setResult(String result) {
        this.result = result;
    }

    public String getFromTerminal() {
        return fromTerminal;
    }

    public void setFromTerminal(String fromTerminal) {
        this.fromTerminal = fromTerminal;
    }

    public String getResultDesc() {
        return resultDesc;
    }

    public void setResultDesc(String resultDesc) {
        this.resultDesc = resultDesc;
    }

    public String getOld_content() {
        return old_content;
    }

    public void setOld_content(String old_content) {
        this.old_content = old_content;
    }

    public String getLink_type() {
        return link_type;
    }

    public void setLink_type(String link_type) {
        this.link_type = link_type;
    }

    public long getLink_id() {
        return link_id;
    }

    public void setLink_id(long link_id) {
        this.link_id = link_id;
    }

    public long getOld_link_id() {
        return old_link_id;
    }

    public void setOld_link_id(long old_link_id) {
        this.old_link_id = old_link_id;
    }

    public String getOperateAuditType() {
        return operateAuditType;
    }

    public void setOperateAuditType(String operateAuditType) {
        this.operateAuditType = operateAuditType;
    }

    public String getOperateAccount() {
        return operateAccount;
    }

    public void setOperateAccount(String operateAccount) {
        this.operateAccount = operateAccount;
    }

    public long getLogOperator() {
        return logOperator;
    }

    public void setLogOperator(long logOperator) {
        this.logOperator = logOperator;
    }

    public long getLogTargetid() {
        return logTargetid;
    }

    public void setLogTargetid(long logTargetid) {
        this.logTargetid = logTargetid;
    }

    public boolean getLogIgnore() {
        return logIgnore;
    }

    public void setLogIgnore(boolean logIgnore) {
        this.logIgnore = logIgnore;
    }

    public Boolean getEsFunction() {
        return esFunction;
    }

    public void setEsFunction(Boolean esFunction) {
        this.esFunction = esFunction;
    }

}

