package com.engine.salary.elog.entity.dto;

import com.engine.salary.elog.annotation.ElogDetailTable;
import com.engine.salary.elog.annotation.ElogField;
import com.engine.salary.elog.enums.ElogConsts;
import com.engine.salary.elog.annotation.ApiModel;
import com.engine.salary.elog.annotation.ApiModelProperty;

import java.io.Serializable;
import java.util.Date;

/**
 * @ClassName: ValueChangeBean
 * @Description 值变化实体类（自动将表字段值变更类转换为该类）
 * @Author tanghj
 * @Date 2021/3/9 11:06
 */
@ElogDetailTable(module = ElogConsts.BASE_TABLE)
@ApiModel("值变化实体类")
public class LoggerDetailContext<T> implements Serializable {

    private static final long serialVersionUID = 15869325700230992L;

    @ElogField(comment = "ID", dataType = DataTypeEnum.BIGINT, isKey = true)
    @ApiModelProperty("ID")
    private long id;

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 36,comment = "日志UUID")
    @ApiModelProperty("同一个bean转换的数据，uuid相同，作为区分标识")
    private String uuid = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 36,comment = "主表id")
    @ApiModelProperty("主表id")
    private String mainid = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 50,comment = "数据id")
    @ApiModelProperty("数据id")
    private String dataid = "";

    /**
     * 当作为主表，belongDataId不赋值
     */
    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 50,comment = "主表数据id")
    @ApiModelProperty("主表数据id")
    private String belongDataid = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 200,comment = "表名")
    @ApiModelProperty("表名")
    private String tableName ="";

    @ElogField(dataType = DataTypeEnum.VARCHAR, defaultValue = "-1", length = 50,comment = "表名labelid")
    @ApiModelProperty("表名labelid")
    private String tableNameLabelId = "-1";
    @ElogField(dataType = DataTypeEnum.VARCHAR, defaultValue="",isNull = false,length = 50,comment = "对应数据库的表")
    @ApiModelProperty("对应数据库的表")
    private String tableNameDesc = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 200,comment = "字段名")
    @ApiModelProperty("字段名")
    private String fieldName ="";

    @ElogField(dataType = DataTypeEnum.VARCHAR, defaultValue = "-1", length = 50,comment = "字段名labelid")
    @ApiModelProperty("字段名labelid")
    private String fieldNameLabelId = "-1";

    @ElogField(dataType = DataTypeEnum.LONGTEXT,comment = "更新后的值")
    @ApiModelProperty("更新后的值")
    private String newValue = "";

    @ElogField(dataType = DataTypeEnum.LONGTEXT,comment = "更新前的值")
    @ApiModelProperty("更新前的值")
    private String oldValue = "";

    @ElogField(dataType = DataTypeEnum.LONGTEXT,comment = "更新后的显示值")
    @ApiModelProperty("更新后的显示值")
    private String newRealValue = "";

    @ElogField(dataType = DataTypeEnum.LONGTEXT,comment = "更新前的显示值")
    @ApiModelProperty("更新前的显示值")
    private String oldRealValue = "";

    @ElogField(dataType = DataTypeEnum.VARCHAR, length = 200, comment = "字段名")
    @ApiModelProperty("字段描述")
    private String fieldDesc = "";

    @ElogField(dataType = DataTypeEnum.INT, comment = "字段名")
    @ApiModelProperty("字段展示顺序")
    private int showorder = 0;

    @ApiModelProperty("自定义字段")
    private T customDetailInfo;

    @ElogField(dataType = DataTypeEnum.DATETIME, defaultValue = "CURRENT_TIMESTAMP", comment = "创建时间")
    @ApiModelProperty("创建时间")
    private Date create_time;

    @ElogField(dataType = DataTypeEnum.DATETIME, defaultValue = "CURRENT_TIMESTAMP", comment = "修改时间")
    @ApiModelProperty("修改时间")
    private Date update_time;

    @ElogField(dataType = DataTypeEnum.BIGINT, defaultValue = "-1", comment = "创建人id")
    @ApiModelProperty("创建人id")
    private long creator;

    @ElogField(dataType = DataTypeEnum.INT, defaultValue = "0" , comment = "是否删除")
    @ApiModelProperty("是否删除")
    private int delete_type;

    @ElogField(dataType = DataTypeEnum.VARCHAR,length = 10,comment = "租户id")
    @ApiModelProperty("租户id")
    private String tenant_key = "";

    @ElogField(dataType = DataTypeEnum.INT, defaultValue = "0" , comment = "是否明细字段")
    @ApiModelProperty("是否明细字段")
    private int isDetail = 0;


    private String cusValus;

    public String getCusValus() {
        return cusValus;
    }

    public void setCusValus(String cusValus) {
        this.cusValus = cusValus;
    }



    public long getId() {
        return id;
    }

    public void setId(long id) {
        this.id = id;
    }

    public String getMainid() {
        return mainid;
    }

    public void setMainid(String mainid) {
        this.mainid = mainid;
    }

    public String getUuid() {
        return uuid;
    }

    public void setUuid(String uuid) {
        this.uuid = uuid;
    }

    public String getTableName() {
        return tableName;
    }

    public void setTableName(String tableName) {
        this.tableName = tableName;
    }

    public String getFieldName() {
        return fieldName;
    }

    public void setFieldName(String fieldName) {
        this.fieldName = fieldName;
    }

    public String getNewValue() {
        return newValue;
    }

    public void setNewValue(String newValue) {
        this.newValue = newValue;
    }

    public String getOldValue() {
        return oldValue;
    }

    public void setOldValue(String oldValue) {
        this.oldValue = oldValue;
    }

    public String getFieldDesc() {
        return fieldDesc;
    }

    public void setFieldDesc(String fieldDesc) {
        this.fieldDesc = fieldDesc;
    }

    public int getShoworder() {
        return showorder;
    }

    public void setShoworder(int showorder) {
        this.showorder = showorder;
    }

    public T getCustomDetailInfo() {
        return customDetailInfo;
    }

    public void setCustomDetailInfo(T customDetailInfo) {
        this.customDetailInfo = customDetailInfo;
    }

    public String getDataid() {
        return dataid;
    }

    public void setDataid(String dataid) {
        this.dataid = dataid;
    }

    public String getBelongDataid() {
        return belongDataid;
    }

    public void setBelongDataid(String belongDataid) {
        this.belongDataid = belongDataid;
    }

    public Date getCreate_time() {
        return create_time;
    }

    public void setCreate_time(Date create_time) {
        this.create_time = create_time;
    }

    public Date getUpdate_time() {
        return update_time;
    }

    public void setUpdate_time(Date update_time) {
        this.update_time = update_time;
    }

    public long getCreator() {
        return creator;
    }

    public void setCreator(long creator) {
        this.creator = creator;
    }

    public int getDelete_type() {
        return delete_type;
    }

    public void setDelete_type(int delete_type) {
        this.delete_type = delete_type;
    }

    public String getTenant_key() {
        return tenant_key;
    }

    public void setTenant_key(String tenant_key) {
        this.tenant_key = tenant_key;
    }

    public int getIsDetail() {
        return isDetail;
    }

    public void setIsDetail(int isDetail) {
        this.isDetail = isDetail;
    }

    public String getNewRealValue() {
        return newRealValue;
    }

    public void setNewRealValue(String newRealValue) {
        this.newRealValue = newRealValue;
    }

    public String getOldRealValue() {
        return oldRealValue;
    }

    public void setOldRealValue(String oldRealValue) {
        this.oldRealValue = oldRealValue;
    }

    public String getTableNameLabelId() {
        return tableNameLabelId;
    }

    public void setTableNameLabelId(String tableNameLabelId) {
        this.tableNameLabelId = tableNameLabelId;
    }

    public String getFieldNameLabelId() {
        return fieldNameLabelId;
    }

    public void setFieldNameLabelId(String fieldNameLabelId) {
        this.fieldNameLabelId = fieldNameLabelId;
    }

    public String getTableNameDesc() {
        return tableNameDesc;
    }

    public void setTableNameDesc(String tableNameDesc) {
        this.tableNameDesc = tableNameDesc;
    }



}
