package com.engine.salary.elog.threadlocal;

import com.alibaba.fastjson.JSONObject;
import com.engine.salary.elog.entity.dto.LoggerContext;

import java.util.ArrayList;
import java.util.List;

/**
 * @Date: 2022/5/18 20:26
 * @Author: deli.xu
 * @Description:
 **/
public class ElogThreadLocal {

    private static final ThreadLocal<List<LoggerContext>> loggerContextList = new ThreadLocal<>();
    private static final ThreadLocal<JSONObject> requestBody = new ThreadLocal<>();

    public static LoggerContext currentLoggerContext() {
        List<LoggerContext> list = loggerContextList.get();

        if(list == null || list.size() == 0) {
            return null;
        } else {
            return list.get(list.size() -1);
        }
    }

    public static void addLoggerContext(LoggerContext loggerContext) {
        if(loggerContext == null)
            return;
        List<LoggerContext> list = loggerContextList.get();
        if(list == null) {
            list = new ArrayList<>();
            loggerContextList.set(list);
        }

        list.add(loggerContext);

    }

    public static List<LoggerContext> getLoggerContextList() {
        return loggerContextList.get();
    }

    public static void clear() {
        loggerContextList.set(null);
    }

    public static void removeLast() {
        List<LoggerContext> list = loggerContextList.get();

        if(list != null) {
            int size = list.size();
            if(size > 0)
                list.remove(size - 1);
        }
    }

    public static void remove() {
        loggerContextList.remove();
    }

    public static JSONObject getRequestBody() {
        return requestBody.get();
    }

    public static void setRequestBody(JSONObject json) {
        requestBody.set(json);
    }

    public static void removeRequestBody() {
        requestBody.remove();
    }
}
