package com.engine.salary.elog.util;


import com.engine.salary.elog.entity.dto.LoggerContext;
import com.engine.salary.elog.enums.FromTerminalType;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.servlet.http.HttpServletRequest;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;

/**
 * @Date: 2022/5/18 23:34
 * @Author: deli.xu
 * @Description: elog服务工具类
 **/
public class ElogSeviceUtils {

    private static final Logger logger = LoggerFactory.getLogger(ElogSeviceUtils.class);
    
    /*@Autowired
    private ComInfoCache comInfoCache;
    private static ComInfoCache baseComInfoCache;

    @PostConstruct
    public void init() {
        baseComInfoCache = comInfoCache;
    }*/

    private static final String TABLE_SPACER = "_";
    private static final String TABLE_SUFFIX = "logs";
    private static final String DETAIL_TABLE_SUFFIX = "_detail";
    public static final String BASE_TABLE = "BASE_ELOG_TABLE";


    public static String getTableName(String module, String function) {
        return getTableName(module, function, false);
    }

    public static String getTableName(String module, String function, boolean isDetail) {
        String tablename = module + TABLE_SPACER + function + TABLE_SUFFIX + (isDetail ? DETAIL_TABLE_SUFFIX : "");
//        SecurityUtil.sqlCheck(tablename);
        return tablename;
    }

    /**
     * String 转枚举
     *
     * @param c
     * @param string
     * @param <T>
     * @return
     */
    public static <T extends Enum<T>> T getEnumFromString(Class<T> c, String string) {
        if (c != null && string != null) {
            try {
                return Enum.valueOf(c, string.trim().toUpperCase());
            } catch (IllegalArgumentException ex) {
            }
        }
        return null;
    }

    /**
     * 获取ip地址
     *
     * @param request
     * @return
     */
    public static String getIp(HttpServletRequest request) {
        String ipAddress = request.getHeader("x-forwarded-for");
        String unknown = "unknown";
        if (ipAddress == null || ipAddress.length() == 0 || unknown.equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getHeader("Proxy-Client-IP");
        }
        if (ipAddress == null || ipAddress.length() == 0 || unknown.equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getHeader("WL-Proxy-Client-IP");
        }
        if (ipAddress == null || ipAddress.length() == 0 || unknown.equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getRemoteAddr();
            String benji = "127.0.0.1";
            String bj = "0:0:0:0:0:0:0:1";
            if (benji.equals(ipAddress) || bj.equals(ipAddress)) {
                ///根据网卡取本机配置的IP
                InetAddress inet = null;
                try {
                    inet = InetAddress.getLocalHost();
                } catch (UnknownHostException e) {
                    logger.error("UnknownHostException", e);
                }
                if (inet != null) {
                    ipAddress = inet.getHostAddress();
                }
            }
        }
        ///对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
        int i = 15;
        String s = ",";
        if (ipAddress != null && ipAddress.length() > i) {
            if (ipAddress.indexOf(s) > 0) {
                ipAddress = ipAddress.substring(0, ipAddress.indexOf(","));
            }
        }
        return ipAddress;
    }

    /**
     * 获取设备信息
     *
     * @param request
     * @return
     */
    public static String getDevice(HttpServletRequest request) {
        return request.getHeader("User-Agent");
    }

    /**
     * 获取来自终端的信息
     *
     * @param context
     * @param request
     * @return
     */
    public static void getFromTerminal(LoggerContext context, HttpServletRequest request) {
        String fromTerminal = context.getFromTerminal();
        if (StringUtils.isEmpty(fromTerminal)) {
            String device = getDevice(request);
            String setFT = "";
            if (StringUtils.isNotEmpty(device)) {
                context.setFromTerminal(getFromTerminal(device));
            }
        }
    }


    private static String getTraceId(HttpServletRequest request) {
        String traceId = request.getHeader("traceId");
        if (StringUtils.isNotBlank(traceId)) {
            //System.out.println("traceId:====="+traceId);
            return traceId;
        }
        return "";
    }

    /**
     * @param request
     * @return localhost:9080/api/fs/demo/updateReport
     */
    public static String getRequestUrl(HttpServletRequest request) {
        if (Objects.isNull(request) || Objects.isNull(request.getRequestURL())) {
            return null;
        }
        return request.getRequestURL().toString();
    }

    /**
     * @param request
     * @return /api/fs/demo/updateReport
     */
    public static String getRequestUri(HttpServletRequest request) {
        return request.getRequestURI();
    }

    /**
     * @param request
     * @return GET/POST
     */
    public static String getRequestMethod(HttpServletRequest request) {
        return request.getMethod();
    }


//    /**
//     * 获取当前方法中的日志实体类
//     * @return
//     */
//    public static LoggerContext currentElogContext() {
//        return ElogThreadLocal.currentLoggerContext();
//    }


    public static int getIntValue(String v) {
        return getIntValue(v, -1);
    }

    public static int getIntValue(String v, int def) {
        try {
            return Integer.parseInt(v);
        } catch (Exception var3) {
            return def;
        }
    }

    public static long getLongValue(String v) {
        return getLongValue(v, -1l);
    }

    public static long getLongValue(String v, long def) {
        try {
            return Long.parseLong(v);
        } catch (Exception var3) {
            return def;
        }
    }

    public static boolean isLongValue(String v) {
        try {
            Long.parseLong(v);
        } catch (Exception e) {
            return false;
        }
        return true;
    }


//    /**
//     * 获取request请求
//     *
//     * @return
//     */
//    public static HttpServletRequest getRequest() {
//        return ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
//    }

    /**
     * 将对象转换程字符串
     *
     * @param obj 目标对象
     * @param def 如果为null时的默认值
     * @return
     */
    public static String null2String(Object obj, String def) {
        return obj == null ? def : obj.toString();
    }

    public static String null2String(Object obj) {
        return null2String(obj, "");
    }

    public static int getIntValue(Object obj, int def) {
        try {
            return Integer.parseInt(null2String(obj));
        } catch (Exception ex) {
            return def;
        }
    }

//    public static Page getPage() {
//        HttpServletRequest request = getRequest();
//        String num = request.getParameter("pageNum");
//        String size = request.getParameter("pageSize");
//
//        Page page = new Page(ElogSeviceUtils.getIntValue(num, 1), ElogSeviceUtils.getIntValue(size, 10));
//        return page;
//    }

//    public static void main(String[] args) {
//        //System.out.println(getTableName("select", "from"));
//        //DataTypeEnum columnTypeEnum = getEnumFromString(DataTypeEnum.class, "varchar");
//    }

    public static List switchString(List<Map> list) {
        if (list != null) {
            return list.stream().map(m -> {
                Set<Map.Entry> en = m.entrySet();
                for (Map.Entry entry : en) {
                    if (entry.getValue() != null) {
                        entry.setValue(String.valueOf(entry.getValue()));
                    }
                }
                return m;
            }).collect(Collectors.toList());
        } else {
            return list;
        }
    }

    public static List switchComplexString(List<Map> list) {
        if (list != null) {
            return list.stream().map(m -> {
                Iterator<Map.Entry> iterator = m.entrySet().iterator();
                while (iterator.hasNext()) {
                    Map.Entry next = iterator.next();
                    if (next.getValue() != null) {
                        if (next.getValue() instanceof Map && "parmas".equals(next.getKey())) {
                            next.setValue(String.valueOf(next.getValue()));
                        }
                        if (next.getValue() instanceof List) {
                        } else {
                            next.setValue(String.valueOf(next.getValue()));
                        }
                    }
                }
                return m;
            }).collect(Collectors.toList());
        } else {
            return list;
        }
    }

    public static String getTenantKey() {
        return "all_teams";
    }

    public static String getEmployeeId() {
        return "";
    }

//    public static String getUserName() {
//
//            String employeeId = getEmployeeId();
//            if (StringUtils.isNotEmpty(employeeId)) {
//                SimpleEmployee simpleEmployee = null;
//                try {
//                    HrmCommonUtil hrmCommonUtil = (HrmCommonUtil) ApplicationContextProvider.getBean("hrmCommonUtil");
//                    simpleEmployee = hrmCommonUtil.getSimpleEmployee(ElogSeviceUtils.getLongValue(employeeId));
//                } catch (Exception e) {
//                    logger.error("Exception", e);
//                }
//                if (simpleEmployee != null) {
//                    if (StringUtils.isNotBlank(simpleEmployee.getUsername())) {
//                        return simpleEmployee.getUsername();
//                    }
//                }
//            }
//
//
//        return "";
//    }

//    /**
//     * 获取rpc信息（客户端ip和来源设备）
//     *
//     * @param context
//     */
//    public static void initRpcInfo(LoggerContext context) {
//        String device = "";
//        String clientIp = "";
//        String traceId = "";
//        try {
//            device = RpcContext.getContext().getAttachment(EteamsConstant.DEVICE);
//            clientIp = RpcContext.getContext().getAttachment(EteamsConstant.CLIENT_IP);
//            traceId = RpcContext.getContext().getAttachment(ApmConstant.TRACE_ID);
//            logger.info("rpc调用获取到 device:{},clientIp:{},traceId:{}", device, clientIp, traceId);
//        } catch (Exception e) {
//            logger.error("Exception", e);
//        }
//        if (StringUtils.isEmpty(context.getDevice()) && StringUtils.isNotEmpty(device)) {
//            context.setDevice(device);
//        }
//        if (StringUtils.isEmpty(context.getFromTerminal()) && StringUtils.isNotEmpty(device)) {
//            context.setFromTerminal(getFromTerminal(device));
//        }
//
//        if (StringUtils.isEmpty(context.getClientIp()) && StringUtils.isNotEmpty(clientIp)) {
//            context.setClientIp(clientIp);
//        }
//        if (StringUtils.isEmpty(context.getBelongMainId()) && StringUtils.isNotEmpty(traceId)) {
//            context.setBelongMainId(traceId);
//        }
//    }

    public static String getFromTerminal(String device) {
        String setFT = "";
        if (StringUtils.isNotEmpty(device)) {
            if (!device.contains("wxwork") && device.contains("MicroMessenger")) {//来自微信
                setFT = FromTerminalType.MICO_MSG.getCode();
            } else if (device.contains("wxwork") && device.contains("MicroMessenger")) {//企业微信
                setFT = FromTerminalType.WECHAT.getCode();
            } else if (device.contains("iPhone") && device.contains("Mac")) {//来自 Iphone
                setFT = FromTerminalType.IOS.getCode();
            } else if (device.contains("Mac OS") && !device.contains("iPhone") && device.contains("weapp-pc")) {//来自 mac_client
                setFT = FromTerminalType.MAC_CLIENT.getCode();//mac_client
            } else if (!device.contains("wxwork") && device.contains("Mobile")) {//移动端
                setFT = FromTerminalType.H5.getCode();
            } else if (device.contains("Android") && !device.contains("wxwork")) {//来自安卓 包含安卓并且不包含微信
                setFT = FromTerminalType.ANDROID.getCode();
            } else {//pc
                setFT = FromTerminalType.PC.getCode();
            }
        }
        return setFT;
    }

    /**
     * sql连接条件注入sql
     *
     * @param condition
     * @return
     */
    public static String checkConditionSql(String condition) {
        if ("AND".equalsIgnoreCase(condition) || "OR".equalsIgnoreCase(condition)) {
            return condition;
        }
        return "AND";
    }

    public static String checkTypeSql(String type) {
        if ("LIKE".equalsIgnoreCase(type) || "IN".equalsIgnoreCase(type) || "!<>".equalsIgnoreCase(type) || "!=".equalsIgnoreCase(type) || "BETWEEN".equalsIgnoreCase(type) || "IS NULL".equalsIgnoreCase(type) || "IS NULL".equalsIgnoreCase(type) || "=".equalsIgnoreCase(type) || "IS NOT NULL".equalsIgnoreCase(type)) {
            return type;
        }
        return "=";
    }

    /**
     * sql条件防止注入
     *
     * @param value
     * @return
     */
    public static String checkValSql(String value) {
        if (StringUtils.isBlank(value)) {
            return "";
        }
        Pattern p = Pattern.compile("\\s+");
        Matcher m = p.matcher(value);
        String val = m.replaceAll(" ");
        String[] keywords = {"master ", "truncate ", "declare ", "alert ", "create ", "drop ", " version", "show ", "table ", "index ", "insert ", "into ", "from ", "select ", "delete ", "update ", "mid ", "master ", "char ", "union "};

        String replaceStr = val.replaceAll(" ", "");
        if (replaceStr.contains("1=1") || replaceStr.contains(";")) {
            return "-1";
        }
        int count = 0;
        String filterVal = "";
        for (String keyddlword : keywords) {
            if (val.toLowerCase().contains(keyddlword)) {
                count++;
                if (count == 1) {
                    filterVal = keyddlword;
                }
            }
        }

        if (count > 2) {
            return filterVal;
        }
//        value = SecurityUtil.ecodeForSql(value);

        return value;
    }

    /**
     * 压缩工具类-
     *
     * @param str
     * @return desc:version 0.0.1 基于jdk自带 GZIP 压缩。最后转成base64。
     * 市面上有其他压缩像jdk 的 deflate 可以设置压缩级别，但是都是主动的，需要改业务方法，
     * snappy 压缩适用于大数据压缩。大数据量比较快 hadoop首选，但是压缩后比例比较大。
     * xz 下的 压缩比率大，但是解压比较慢-不提倡，空间换时间了
     * common下的压缩其实和jdk差不多，网上说优于jdk，但是相差不大。
     * weaver 压缩基于jdk
     */
    public static String compress(String str) {
        if (str == null || str.trim().length() == 0) {
            return str;
        }
        try (ByteArrayOutputStream out = new ByteArrayOutputStream(); GZIPOutputStream gzip = new GZIPOutputStream(out)) {
            gzip.write(str.getBytes());
            gzip.close();
            return Base64.getEncoder().encodeToString(out.toByteArray());
        } catch (Exception e) {
            logger.error("压缩失败", e.getMessage());
            return str;
        }

    }


    /**
     * 解压缩
     *
     * @param str
     * @return
     */
    public static String uncompress(String str) {
        byte[] decode = Base64.getDecoder().decode(str);
        if (str == null || str.trim().length() == 0) {
            return str;
        }
        try (ByteArrayOutputStream out = new ByteArrayOutputStream(); ByteArrayInputStream in = new ByteArrayInputStream(decode)) {
            GZIPInputStream ungzip = new GZIPInputStream(in);
            byte[] buffer = new byte[2048];
            int n;
            while ((n = ungzip.read(buffer)) >= 0) {
                out.write(buffer, 0, n);
            }
            return new String(out.toByteArray());
        } catch (Exception e) {
            logger.error("解缩失败：{}", e.getMessage());
            return str;
        }
    }

    public static Boolean checkIsNumber(Object obj) {
        if (obj == null) {
            return false;
        }

        return StringUtils.isNumeric(obj.toString());

    }
}
