package com.engine.salary.elog.web;

import com.engine.common.util.ServiceUtil;
import com.engine.salary.elog.entity.param.ELogGetLogParam;
import com.engine.salary.elog.entity.param.GetDetailChangesParam;
import com.engine.salary.elog.service.ILoggerTableService;
import com.engine.salary.elog.service.impl.LoggerTableService;
import com.engine.salary.elog.util.ResponseResult;
import com.engine.salary.elog.util.page.PageInfo;
import io.swagger.v3.oas.annotations.parameters.RequestBody;
import weaver.hrm.HrmUserVarify;
import weaver.hrm.User;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import java.util.List;
import java.util.Map;

public class LoggerTableController {

    private ILoggerTableService getLoggerTableService(User user) {
        return ServiceUtil.getService(LoggerTableService.class, user);
    }

    /**
     * 获取日志
     *
     * @param request
     * @param response
     * @param data
     * @return
     */
    @POST
    @Path("/getLogs")
    @Produces(MediaType.APPLICATION_JSON)
    public String getLogs(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody String data) {
        User user = HrmUserVarify.getUser(request, response);
        return new ResponseResult<String, PageInfo>(user).run(getLoggerTableService(user)::queryLogs, data);
    }

    /**
     * 获取日志（卡片）
     *
     * @param request
     * @param response
     * @param data
     * @return
     */
    @POST
    @Path("/getCardLogs")
    @Produces(MediaType.APPLICATION_JSON)
    public String carddatas(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody String data) {
        User user = HrmUserVarify.getUser(request, response);
        return new ResponseResult<String, List<Map<String, Object>>>(user).run(getLoggerTableService(user)::queryCardLogList, data);
    }

    /**
     * 获取单条操作记录的更新明细
     * @param request
     * @param response
     * @param param
     * @return
     */
    @POST
    @Path("/getDetailChanges")
    @Produces(MediaType.APPLICATION_JSON)
    public String getDetailChanges(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody GetDetailChangesParam param) {
        User user = HrmUserVarify.getUser(request, response);
        return new ResponseResult<GetDetailChangesParam, List>(user).run(getLoggerTableService(user)::getDetailChanges, param);
    }
//    @ApiOperation("获取单条操作记录的更新明细")
//    @POST
//    @Path("/getDetailChanges")
//    @Produces(MediaType.APPLICATION_JSON)
//    public String getDetailChanges(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody String data) {
//        User user = HrmUserVarify.getUser(request, response);
//        return new ResponseResult<String, List<Map<String, Object>>>(user).run(getLoggerTableService(user)::getDetailChanges, data);
//    }
//
//    /**
//     * 获取单条操作记录的更新明细(分页)
//     *
//     * @param module            服务（模块）名
//     * @param function          方法名
//     * @param mainid            主键id
//     * @param detailTransMethod 转换方法
//     * @param current           页码
//     * @param pageSize          每页条数
//     * @return WeaTable
//     */
//    @ApiOperation("获取单条操作记录的更新明细(分页)")
//    @RequestMapping(path = "getDetailChangePages", method = {RequestMethod.GET, RequestMethod.POST})
//    @WeaPermission(publicPermission = true)
//    public WeaResult<WeaTable> getDetailChangePages(@PathVariable("module") @ApiParam("服务（模块）名") String module,
//                                                    @PathVariable("function") @ApiParam("方法名") String function,
//                                                    @RequestParam("mainid") @ApiParam("主键id") String mainid,
//                                                    @RequestParam("detailTransMethod") @ApiParam("转换方法") String detailTransMethod,
//                                                    @RequestParam("page") @ApiParam("页码") String current,
//                                                    @RequestParam("pageSize") @ApiParam("每页条数") String pageSize) {
//        return WeaResult.success(getLoggerTableService(user).getDetailChangePages(module, function, mainid, detailTransMethod, current, pageSize));
//    }

    /**
     * 获取日志列表
     *
     * @param request
     * @param response
     * @param param
     * @return
     */
    @POST
    @Path("/datas")
    @Produces(MediaType.APPLICATION_JSON)
    public String datas(@Context HttpServletRequest request, @Context HttpServletResponse response, @RequestBody ELogGetLogParam param) {
        User user = HrmUserVarify.getUser(request, response);
        return new ResponseResult<ELogGetLogParam, List>(user).run(getLoggerTableService(user)::queryLogList, param);
    }


//    @ApiOperation("获取日志总数")
//    @RequestMapping(path = "counts", method = {RequestMethod.GET, RequestMethod.POST})
//    @WeaPermission(publicPermission = true)
//    public WeaResult<Map<String, Object>> counts(@PathVariable("module") @ApiParam("服务（模块）名") String module,
//                                                 @PathVariable("function") @ApiParam("方法名") String function) {
//        return WeaResult.success(getLoggerTableService(user).countLog(module, function), "success");
//    }
//
//    @ApiOperation("获取明细日志列表")
//    @RequestMapping(path = "detaildatas", method = {RequestMethod.GET, RequestMethod.POST})
//    @WeaPermission(publicPermission = true)
//    public WeaResult<List<Map<String, Object>>> detaildatas(@PathVariable("module") @ApiParam("服务（模块）名") String module,
//                                                            @PathVariable("function") @ApiParam("方法名") String function,
//                                                            @Param("pageSize") @ApiParam("每页多少数据") String pageSize,
//                                                            @Param("current") @ApiParam("当前页") String current,
//                                                            @Param("mainId") @ApiParam("主表id") String mainId,
//                                                            @ApiParam("查询条件") String condition) {
//
//        if ("datasecurity".equals(module) && "auditLog".equals(function)) {
//            try {
//                ILoggerTableService tableService = ApplicationContextProvider.getBean("auditLogService", ILoggerTableService.class);
//                if (tableService != null) {
//                    List<Map<String, Object>> list = tableService.queryDetailLogList(module, function, current, pageSize, condition, mainId);
//                    return WeaResult.success(list);
//                }
//            } catch (Exception e) {
//
//            }
//        }
//        List<Map<String, Object>> list = getLoggerTableService(user).queryDetailLogList(module, function, current, pageSize, condition, mainId);
//        return WeaResult.success(list);
//    }
//
//    @ApiOperation("获取日志总数")
//    @RequestMapping(path = "detailcounts", method = {RequestMethod.GET, RequestMethod.POST})
//    @WeaPermission(publicPermission = true)
//    public WeaResult<Map<String, Object>> detailcounts(@PathVariable("module") @ApiParam("服务（模块）名") String module,
//                                                       @PathVariable("function") @ApiParam("方法名") String function,
//                                                       @Param("mainId") @ApiParam("主表id") String mainId) {
//        return WeaResult.success(getLoggerTableService(user).countDestailLog(module, function, mainId), "success");
//    }
//
//    @ApiOperation("根据traceId获取链路列表")
//    @RequestMapping(path = "queryLogTraceInfo", method = {RequestMethod.GET, RequestMethod.POST})
//    @WeaPermission(publicPermission = true)
//    public WeaResult<WeaTable> queryLogTraceInfo(@PathVariable("module") @ApiParam("服务（模块）名") String module,
//                                                 @PathVariable("function") @ApiParam("方法名") String function,
//                                                 @Param("traceId") @ApiParam("traceId") String traceId,
//                                                 @Param("currentPage") @ApiParam("currentPage") Integer currentPage,
//                                                 @Param("pageSize") @ApiParam("pageSize") Integer pageSize,
//                                                 @Param("traceTransMethod") @ApiParam("traceTransMethod") String traceTransMethod
//    ) {
//
//        if ("datasecurity".equals(module) && "auditLog".equals(function)) {
//            try {
//                ILoggerTableService tableService = ApplicationContextProvider.getBean("auditLogService", ILoggerTableService.class);
//                if (tableService != null) {
//                    return WeaResult.success(tableService.queryElogTraceInfo(traceId, module, function, currentPage, pageSize, traceTransMethod), "success");
//                }
//            } catch (Exception e) {
//
//            }
//        }
//
//        return WeaResult.success(getLoggerTableService(user).queryElogTraceInfo(traceId, module, function, currentPage, pageSize, traceTransMethod), "success");
//    }
//
//    @ApiOperation("日志下载")
//    @RequestMapping(path = "downloadLog", method = {RequestMethod.GET, RequestMethod.POST})
//    @WeaPermission(publicPermission = true)
//    public WeaResult<BatchDocumentMessage> downloadLog(@RequestBody(required = false) @ApiParam("数据") String data) {
//
//        BatchDocumentMessage message = getLoggerTableService(user).downloadLog(data);
//        return WeaResult.success(message);
//    }
}
